/**
******************************************************************************
* @file    uart_usb.c
* @author  IC Applications Department
* @version  V0.8
* @date  2019_08_02
* @brief   This file provides all the uart_usb firmware functions.
******************************************************************************
* @copy
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, HOLOCENE SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
* <h2><center>&copy; COPYRIGHT 2016 HOLOCENE</center></h2>
*/ 
#include "sys.h"
#include "uart_usb.h"
#include "HAl_device.h"

//´,֧printf,Ҫѡuse MicroLIB
#ifdef __GNUC__

#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)

#endif

#ifdef USE_IAR
PUTCHAR_PROTOTYPE
{
    while((UART1->CSR & UART_IT_TXIEN) == 0); //ѭ,ֱ
    UART1->TDR = (ch & (uint16_t)0x00FF);
    return ch;
}

#else
#pragma import(__use_no_semihosting)
//׼Ҫֺ֧
struct __FILE
{
    int handle;

};

FILE __stdout;
//_sys_exit()Աʹðģʽ
_sys_exit(int x)
{
    x = x;
}
//ضfputc
int fputc(int ch, FILE *f)
{
    while((UART1->CSR & UART_IT_TXIEN) == 0); //ѭ,ֱ
    UART1->TDR = (ch & (uint16_t)0x00FF);
    return ch;
}

#endif

//////////////////////////////////////////////////////////////////


void uart_sendByte(unsigned char data)
{
    while((UART1->CSR & UART_IT_TXIEN) == 0); //ѭ,ֱ
    UART1->TDR = data;
}

void uart_sendArray(unsigned char *dataBuf, unsigned int len)
{
    while(len)
    {
        uart_sendByte(*dataBuf);
        len--;
        dataBuf++;
    }
}

#if EN_UART1_RX   //ʹ˽
//1жϷ
//ע,ȡUARTx->SRܱĪĴ
u8 UART_RX_BUF[UART_REC_LEN];     //ջ,UART_REC_LENֽ.
//״̬
//bit15	ɱ־
//bit14	յ0x0d
//bit13~0	յЧֽĿ
u16 UART_RX_STA = 0;     //״̬

//ʼIO 1
//pclk2:PCLK2ʱƵ(Mhz)
//bound:
void UartInit(UART_TypeDef* UARTx,int BaudRate)
{
	UART_InitTypeDef       UART_InitStructure;  
	GPIO_InitTypeDef  GPIO_InitStructure;   
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_UART1, ENABLE);
	
	GPIO_PinAFConfig(GPIOA, GPIO_Pin_9 | GPIO_Pin_10, GPIO_AF_UART_1); //PA9PA10Ϊ1
	
	UART_InitStructure.UART_BaudRate = BaudRate; //
  UART_InitStructure.UART_WordLength = UART_WordLength_8b;//λ
  UART_InitStructure.UART_StopBits = UART_StopBits_1;//ֹͣλ
  UART_InitStructure.UART_Parity = UART_Parity_No ;
  UART_InitStructure.UART_Mode = UART_Mode_Rx | UART_Mode_Tx;//ģʽ
  UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None; 	
	UART_Init(UARTx, &UART_InitStructure);
	UART_Cmd(UARTx, ENABLE);  //UART ģʹ
	
  UART_ClearITPendingBit(UARTx, 0xff); 
	
	UART_ITConfig(UARTx, UART_IT_RXIEN, ENABLE);//ʹܽж
	NVIC_SetPriority(UART1_IRQn,3);
	NVIC_EnableIRQ(UART1_IRQn);
	
	//עʼ˳򿪣ַӡ
	GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_9;   //uart1_tx  pa9
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; // ⸴
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_10;  //uart1_rx  pa10
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU; //   
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}

unsigned int curUartRxLenth = 0;

void UART1_IRQHandler(void)
{

    if((UART1->ISR & UART_IT_RXIEN) != (uint16_t)RESET)	//յ
    {
        UART1->ICR |= 2;//ж //		UART_ClearITPendingBit(UART1,UART_IT_RXIEN);
        UART_RX_BUF[curUartRxLenth] = UART1->RDR;
        curUartRxLenth ++;

        if(curUartRxLenth > 199) {curUartRxLenth = 0;}
    }
}
#endif

/**
* @}
*/

/**
* @}
*/

/**
* @}
*/

/*-------------------------(C) COPYRIGHT 2016 HOLOCENE ----------------------*/
