/**
******************************************************************************
* @file    usbprop.c
* @author  IC Applications Department
* @version  V0.8
* @date  2019_08_02
* @brief   This file provides all the usbprop firmware functions.
******************************************************************************
* @copy
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, HOLOCENE SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
* <h2><center>&copy; COPYRIGHT 2016 HOLOCENE</center></h2>
*/ 

#include "usbprop.h"
#include "usb.h"
#include "HAL_device.h"
#include "uart_usb.h"

unsigned char *pucClassDrcData;


//⴮ڲ
LINE_CODING linecoding =
{
    460800, /* baud rate*/
    0x00,   /* stop bits-1*/
    0x00,   /* parity - none*/
    0x08    /* no. of bits 8*/
};


/********************************************************************************************************
**Ϣ USBCDC_senddata(unsigned char *data,unsigned short length)
** ת
** unsigned char *data(תָ),unsigned short length(תݳ)
** 
**    ע 
********************************************************************************************************/
void USBCDC_senddata(unsigned char *data, unsigned short length)
{
    if(linecoding.datatype == 7)
    {
        while(length)
        {
            *data &= 0x7f;
            length--;
            data++;
        }
    }
    else if(linecoding.datatype == 8)							//
    {
        while(length)
        {

        }
    }
}

/********************************************************************************************************
**Ϣ Class_Get_Line_Coding(void)
** CDC Abstract Control Model(⴮) GET_LINE_CODING
** 
** 
**    ע 
********************************************************************************************************/
void Class_Get_Line_Coding(void)      							// bRequest = 0x21
{
    int count;
    switch( usb_running_ctrl_state )
    {
        case    USB_CTRL_SETUP :
            if(req_dir == USB_REQ_DIR_IN)
            {
                pucClassDrcData = (u8*)&linecoding;
                usb_running_ctrl_state = USB_CTRL_IN;
            }
            else
            {
                usb_running_ctrl_state = USB_IDLE;
            }
            break ;

        case    USB_CTRL_IN :

            while(USB->rEP0_CTRL & 0x80);
            for( count = 0; count < 7; count++ )
            {
                USB->rEP0_FIFO = *pucClassDrcData;
                pucClassDrcData ++;
            }
            USB->rEP0_CTRL = 0x87;

            usb_running_ctrl_state = USB_IDLE;
            break ;

        default :
            //		usb_running_ctrl_state = USB_IDLE ;
            break ;
    }
}
unsigned char baud_read [64];
/********************************************************************************************************
**Ϣ Class_Get_Line_Coding(void)
** CDC Abstract Control Model(⴮) SET_LINE_CODING
** 
** 
**    ע 
********************************************************************************************************/
void Class_Set_Line_Coding(void)  // bRequest = 0x20
{
    int count, i;

    switch( usb_running_ctrl_state )
    {
        case    USB_CTRL_SETUP :
            if(req_dir == USB_REQ_DIR_OUT)
            {
                usb_running_ctrl_state = USB_CTRL_OUT;
            }
            else
            {
                usb_running_ctrl_state = USB_IDLE;
            }
            break ;

        case    USB_CTRL_OUT :
            count = USB->rEP0_AVIL;
            for(i = 0; i < count ; i++)
            {
                baud_read[i] = USB->rEP0_FIFO;
            }
            linecoding.bitrate = ((u32)baud_read[0] << 0) | ((u32)baud_read[1] << 8) | ((u32)baud_read[2] << 16) | ((u32)baud_read[3] << 24);
						UartInit(UART1,linecoding.bitrate);
            usb_running_ctrl_state = USB_CTRL_STATUS;
            break;

        default :
            //			usb_running_ctrl_state = USB_IDLE ;
            break ;
    }
}
/********************************************************************************************************
**Ϣ Class_Get_Line_Coding(void)
** CDC Abstract Control Model(⴮) SET_LINE_CODING
** 
** 
**    ע 
********************************************************************************************************/

void Class_Set_Control_Line_State(void)      				// bRequest = 0x22
{
    int count, i;
    unsigned char state_temp = 0;
    state_temp = state_temp;
    switch( usb_running_ctrl_state )
    {
        case    USB_CTRL_SETUP :							//ƴ״̬׶
            if(req_dir == USB_REQ_DIR_OUT)
            {
                usb_running_ctrl_state = USB_CTRL_OUT;
            }
            else
            {
                usb_running_ctrl_state = USB_IDLE;
            }
            break ;
        case    USB_CTRL_OUT :
            count = USB->rEP0_AVIL;
            for(i = 0; i < count ; i++)
            {
                state_temp = USB->rEP0_FIFO;				//˴Ϊ(ƴݽ׶)
            }
            usb_running_ctrl_state = USB_CTRL_STATUS;
            break;

        default :
            {
                //	        usb_running_ctrl_state = USB_IDLE ;
                break ;
            }
    }
}

unsigned char ep1_Tx_dataBuf[2048];			//USB ͻ,ԶС
unsigned int ui_curTxLen = 0;				//ʵʱͳ
unsigned int ep1_Tx_dataLen = 0;			//ܳ(СԶ建С)
unsigned char ep1_tx_flag = 0;				//ͱ־

/********************************************************************************************************
**Ϣ EP1_USB_IN_Data(void)
** USBӷͻͨIN˵㷢
** 
** 
**    ע USBݷʱݳ˵sizeʱҪ
********************************************************************************************************/
void EP1_USB_IN_Data(void)      			// bRequest = 0x22
{
    int count, i;

    if(ep1_tx_flag != 0)						//tx_bufǿ
    {
        while(USB->rEP1_CTRL & 0x80);
        if(ep1_Tx_dataLen > 64)				//USBݲַ
        {
            count = 64;
            ep1_Tx_dataLen -= 64;
        }
        else
        {
            count = ep1_Tx_dataLen;
            ep1_Tx_dataLen = 0;
        }
        for(i = 0; i < count; i++)
        {
            USB->rEP1_FIFO = *(ep1_Tx_dataBuf + ui_curTxLen + i);
        }
        USB->rEP1_CTRL = 0x80 | count;
        if(ep1_Tx_dataLen == 0)
        {
            ep1_tx_flag = 0;
            ui_curTxLen = 0;
            usb_running_ctrl_state = USB_IDLE;
        }
        else
        {
            ui_curTxLen += count;
        }
    }

}
/********************************************************************************************************
**Ϣ UsbVcomSend(unsigned char*Info,unsigned int *infoLenth)
** Ҫ͵дUSBͻ
** unsigned char*Info(ָ),unsigned int *infoLenth(ݳָ)
** return status(0ʾ,1ʾڷ);
**    ע 
********************************************************************************************************/
char UsbVcomSend(unsigned char*Info, unsigned int *infoLenth)
{
    unsigned int i, status = 0;
    if((ep1_tx_flag == 0) && (*infoLenth != 0))
    {
        ep1_Tx_dataLen = *infoLenth;
        *infoLenth = 0;
        for(i = 0; i < ep1_Tx_dataLen; i++)
        {
            ep1_Tx_dataBuf[i] = *(Info + i);
        }
        ep1_tx_flag = 1;
    }
    else
    {
        status = 1;
    }
    return status;
}
unsigned char ep3_rx_flag = 0;				//˵3ձ־
/********************************************************************************************************
**Ϣ EP3_USB_OUT_Data(void)
** USB_OUT˵յЧ,ձ־λ1
** 
** 
**    ע 
********************************************************************************************************/
void EP3_USB_OUT_Data(void)      			// bRequest = 0x22
{
    if (ep3_rx_flag == 0)					//rx_buf
    {
        ep3_rx_flag = 1;
    }
    usb_running_ctrl_state = USB_IDLE;
}
/********************************************************************************************************
**Ϣ UsbVcomRec(unsigned char*rxInfo)
** USB device OUT˵
** unsigned char*rxInfo(Ϣbuf)
** 
**    ע 
********************************************************************************************************/
unsigned int UsbVcomRec(unsigned char*rxInfo)
{
    unsigned int i, temp_len;
    if(ep3_rx_flag == 1)					//bufǿ
    {
        temp_len = USB->rEP3_AVIL & 0x7f;
        for(i = 0; i < temp_len; i++)
        {
            *(rxInfo + i) = USB->rEP3_FIFO;
        }
        ep3_rx_flag = 0;
    }
    else
    {
        return 0;
    }
    return temp_len;
}

/*-------------------------(C) COPYRIGHT 2016 HOLOCENE ----------------------*/
